/*
 * DS3234S.c
 *
 * Created: 2019/04/25 8:53:24
 *  Author: mNEFO
 */ 

#include <avr/eeprom.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>

#include "common.h"
#include "eeprom.h"
#include "extern.h"
#include "DS3234S.h"

// init SPI
void init_SPI(void)
{
	SPCR = _BV(SPE) | _BV(MSTR) | _BV(CPHA); // SPI Master Mode 0
	SPSR = _BV(SPI2X);
}

// init DS3234S
// init DS3234S must do after read eeprom
void init_DS3234S(void)
{
	//uint8_t eeprom_year;
	uint8_t temp_DS3234S;

	// DS3234S init
	DS3234S_write_bcd(DS3234S_CONTROL_ADDR, 0b00000000); // 1Hz, INT output
	DS3234S_write_bcd(DS3234S_CONTROL_STATUS_ADDR, (DS3234S_STATUS_OSF));

	// check DS3234S data
	DS3234S_read_all();

	//sec
	if((global_DS3234S_sec & 0b00001111) > 9)
	{
		reset_DS3234S();
		return;
	}
	if((global_DS3234S_sec >> 4) > 5)
	{
		reset_DS3234S();
		return;
	}
	temp_DS3234S = (global_DS3234S_sec >> 4) * 10 + (global_DS3234S_sec & 0b00001111);
	if(temp_DS3234S > 59)
	{
		reset_DS3234S();
		return;
	}

	// min
	if((global_DS3234S_min & 0b00001111) > 9)
	{
		reset_DS3234S();
		return;
	}
	if((global_DS3234S_min >> 4) > 5)
	{
		reset_DS3234S();
		return;
	}
	temp_DS3234S = (global_DS3234S_min >> 4) * 10 + (global_DS3234S_min & 0b00001111);
	if(temp_DS3234S > 59)
	{
		reset_DS3234S();
		return;
	}

	// hour
	if(global_DS3234S_hour & DS3234S_AMPM_FLAG) // 12hr system
	{
		if((global_DS3234S_hour & 0b00001111) > 9)
		{
			reset_DS3234S();
			return;
		}
		if(((global_DS3234S_hour >> 4) & 0b00000001) > 1)
		{
			reset_DS3234S();
			return;
		}
		temp_DS3234S = ((global_DS3234S_hour >> 4) & 0b00000001) * 10 + (global_DS3234S_hour & 0b00001111);
		if(temp_DS3234S > 12)
		{
			reset_DS3234S();
			return;
		}
	}
	else // 24hr system
	{
		if((global_DS3234S_hour & 0b00001111) > 9)
		{
			reset_DS3234S();
			return;
		}
		if(((global_DS3234S_hour >> 4) & 0b00000011) > 2)
		{
			reset_DS3234S();
			return;
		}
		temp_DS3234S = ((global_DS3234S_hour >> 4) & 0b00000011) * 10 + (global_DS3234S_hour & 0b00001111);
		if(temp_DS3234S > 24)
		{
			reset_DS3234S();
			return;
		}
	}

	// day
	if(global_DS3234S_day > 7) // 1->Sun 7->Sat
	{
		reset_DS3234S();
		return;
	}

	// date
	if((global_DS3234S_date & 0b00001111) > 9)
	{
		reset_DS3234S();
		return;
	}
	if((global_DS3234S_date >> 4) > 3)
	{
		reset_DS3234S();
		return;
	}
	temp_DS3234S = (global_DS3234S_date >> 4) * 10 + (global_DS3234S_date & 0b00001111);
	if(temp_DS3234S > 31)
	{
		reset_DS3234S();
		return;
	}

	// month
	if((global_DS3234S_month & 0b00001111) > 9)
	{
		reset_DS3234S();
		return;
	}
	if(((global_DS3234S_month >> 4) & 0b00000001) > 1)
	{
		reset_DS3234S();
		return;
	}
	temp_DS3234S = ((global_DS3234S_month >> 4) & 0b00000001) * 10 + (global_DS3234S_month & 0b00001111);
	if(temp_DS3234S > 12)
	{
		reset_DS3234S();
		return;
	}

	// year
	if((global_DS3234S_year & 0b00001111) > 9)
	{
		reset_DS3234S();
		return;
	}
	if((global_DS3234S_year >> 4) > 9)
	{
		reset_DS3234S();
		return;
	}
	temp_DS3234S = (global_DS3234S_year >> 4) * 10 + (global_DS3234S_year & 0b00001111);
	if(temp_DS3234S > 99)
	{
		reset_DS3234S();
		return;
	}
}

// check century flag in month address
void check_century(void)
{
	uint8_t century;

	// read manually, prevent loop
	if((global_DS3234S_month & DS3234S_CENTURY_FLAG) != 0)
	{
		// check year & month & date to prevent startup century flag issue
		if(global_DS3234S_year == 0 || global_DS3234S_month == 1 || global_DS3234S_date == 1)
		{
			century = (global_config_data.century >> 4) * 10 + (global_config_data.century & 0b00001111);
			century++;
			if(century > 99){century = 0;}
			eep_update_century(century);
			// remove flag
			global_DS3234S_month &= (uint8_t)~DS3234S_CENTURY_FLAG;
			// write new month
			DS3234S_CS_PORT &= (uint8_t)~_BV(DS3234S_CS);
			SPDR = DS3234S_MONTH_ADDR  + DS3234S_WRITE;
			loop_until_bit_is_set(SPSR, SPIF);
			SPDR = global_DS3234S_month;
			loop_until_bit_is_set(SPSR, SPIF);
			DS3234S_CS_PORT |= _BV(DS3234S_CS);
		}
		else
		{
			// remove flag
			global_DS3234S_month &= (uint8_t)~DS3234S_CENTURY_FLAG;
			// write new month
			DS3234S_CS_PORT &= (uint8_t)~_BV(DS3234S_CS);
			SPDR = DS3234S_MONTH_ADDR  + DS3234S_WRITE;
			loop_until_bit_is_set(SPSR, SPIF);
			SPDR = global_DS3234S_month;
			loop_until_bit_is_set(SPSR, SPIF);
			DS3234S_CS_PORT |= _BV(DS3234S_CS);
		}
	}
}

// read DS3234S all data
void DS3234S_read_all(void)
{
	DS3234S_CS_PORT &= (uint8_t)~_BV(DS3234S_CS);

	// start from sec
	SPDR = DS3234S_SEC_ADDR + DS3234S_READ;
	loop_until_bit_is_set(SPSR, SPIF);

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_sec = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_min = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_hour = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_day = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_date = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_month = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_year = SPDR;

	DS3234S_CS_PORT |= _BV(DS3234S_CS);

	check_century();
}

// read DS3234S time
void DS3234S_read_time(void)
{
	DS3234S_CS_PORT &= (uint8_t)~_BV(DS3234S_CS);

	// start from sec
	SPDR = DS3234S_SEC_ADDR + DS3234S_READ;
	loop_until_bit_is_set(SPSR, SPIF);

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_sec = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_min = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_hour = SPDR;

	DS3234S_CS_PORT |= _BV(DS3234S_CS);
}

// read DS3234S date
void DS3234S_read_date(void)
{
	DS3234S_CS_PORT &= (uint8_t)~_BV(DS3234S_CS);

	// start from day
	SPDR = DS3234S_DAY_ADDR + DS3234S_READ;
	loop_until_bit_is_set(SPSR, SPIF);

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_day = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_date = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_month = SPDR;

	SPDR = SPI_DUMMY;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_year = SPDR;

	DS3234S_CS_PORT |= _BV(DS3234S_CS);

	check_century();
}

// write DS3234S
void DS3234S_write_bcd(uint8_t addr, uint8_t data)
{
	uint8_t w_addr = 0;
	uint8_t temp;

	temp = (data >> 4) * 10 + (data & 0b00001111);

	switch(addr)
	{
		case DS3234S_SEC_ADDR:
		if(temp < 0 || temp > 59){return;}
		break;

		case DS3234S_MIN_ADDR:
		if(temp < 0 || temp > 59){return;}
		break;

		// this hour must be 24hr format
		case DS3234S_HOUR_ADDR:
		if(temp < 0 || temp > 23){return;}
		break;

		case DS3234S_DAY_ADDR:
		if(temp < 1 || temp > 7){return;}
		break;

		case DS3234S_DATE_ADDR:
		if(temp < 0 || temp > 31){return;}
		break;

		case DS3234S_MONTH_ADDR:
		check_century();
		if(temp < 0 || temp > 12){return;}
		break;

		case DS3234S_YEAR_ADDR:
		if(temp < 0 || temp > 99){return;}
		break;

		case DS3234S_CONTROL_ADDR:
		break;

		case DS3234S_CONTROL_STATUS_ADDR:
		break;

		default:
		return;
	}

	w_addr = addr + DS3234S_WRITE;
	DS3234S_CS_PORT &= (uint8_t)~_BV(DS3234S_CS);
	SPDR = w_addr;
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = data;
	loop_until_bit_is_set(SPSR, SPIF);
	DS3234S_CS_PORT |= _BV(DS3234S_CS);
}

void DS3234S_write_int(uint8_t addr, uint8_t data)
{
	uint8_t w_addr = 0;
	uint8_t str[3];
	uint8_t temp;

	sprintf((char *)str, "%02d", data);
	CHARNUM_TO_NUM(str[0]);
	CHARNUM_TO_NUM(str[1]);
	temp = data;
	data = (str[0] << 4) | str[1];

	switch(addr)
	{
		case DS3234S_SEC_ADDR:
		if(temp < 0 || temp > 59){return;}
		break;

		case DS3234S_MIN_ADDR:
		if(temp < 0 || temp > 59){return;}
		break;

		// this hour must be 24hr format
		case DS3234S_HOUR_ADDR:
		if(temp < 0 || temp > 23){return;}
		break;

		case DS3234S_DAY_ADDR:
		if(temp < 1 || temp > 7){return;}
		break;

		case DS3234S_DATE_ADDR:
		if(temp < 0 || temp > 31){return;}
		break;

		case DS3234S_MONTH_ADDR:
		check_century();
		if(temp < 0 || temp > 12){return;}
		break;

		case DS3234S_YEAR_ADDR:
		if(temp < 0 || temp > 99){return;}
		break;

		case DS3234S_CONTROL_ADDR:
		break;

		case DS3234S_CONTROL_STATUS_ADDR:
		break;

		default:
		return;
	}

	w_addr = addr + DS3234S_WRITE;
	DS3234S_CS_PORT &= (uint8_t)~_BV(DS3234S_CS);
	SPDR = w_addr;
	loop_until_bit_is_set(SPSR, SPIF);
	SPDR = data;
	loop_until_bit_is_set(SPSR, SPIF);
	DS3234S_CS_PORT |= _BV(DS3234S_CS);
}

void DS3234S_write_time_bcd(uint8_t sec, uint8_t min, uint8_t hour)
{
	DS3234S_write_bcd(DS3234S_SEC_ADDR, sec);
	DS3234S_write_bcd(DS3234S_MIN_ADDR, min);
	DS3234S_write_bcd(DS3234S_HOUR_ADDR, hour);
}

void DS3234S_write_date_bcd(uint8_t day, uint8_t date, uint8_t month, uint8_t year)
{
	DS3234S_write_bcd(DS3234S_DAY_ADDR, day);
	DS3234S_write_bcd(DS3234S_DATE_ADDR, date);
	DS3234S_write_bcd(DS3234S_MONTH_ADDR, month);
	DS3234S_write_bcd(DS3234S_YEAR_ADDR, year);
}

void DS3234S_write_time_int(uint8_t sec, uint8_t min, uint8_t hour)
{
	DS3234S_write_int(DS3234S_SEC_ADDR, sec);
	DS3234S_write_int(DS3234S_MIN_ADDR, min);
	DS3234S_write_int(DS3234S_HOUR_ADDR, hour);
}


void DS3234S_write_date_int(uint8_t day, uint8_t date, uint8_t month, uint8_t year)
{
	DS3234S_write_int(DS3234S_DAY_ADDR, day);
	DS3234S_write_int(DS3234S_DATE_ADDR, date);
	DS3234S_write_int(DS3234S_MONTH_ADDR, month);
	DS3234S_write_int(DS3234S_YEAR_ADDR, year);
}

void reset_DS3234S(void)
{
	DS3234S_CS_PORT &= (uint8_t)~_BV(DS3234S_CS);

	SPDR = DS3234S_SEC_ADDR + DS3234S_WRITE;
	loop_until_bit_is_set(SPSR, SPIF);

	// sec
	SPDR = DS3234S_SEC_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_sec = DS3234S_SEC_DEFAULT;

	// min
	SPDR = DS3234S_MIN_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_min = DS3234S_MIN_DEFAULT;

	// hour
	SPDR = DS3234S_HOUR_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_hour = DS3234S_HOUR_DEFAULT;

	// day
	SPDR = DS3234S_DAY_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_day = DS3234S_DAY_DEFAULT;

	// date
	SPDR = DS3234S_DATE_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_date = DS3234S_DATE_DEFAULT;

	// month
	SPDR = DS3234S_MONTH_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_month = DS3234S_MONTH_DEFAULT;

	// year
	SPDR = DS3234S_YEAR_DEFAULT;
	loop_until_bit_is_set(SPSR, SPIF);
	global_DS3234S_year = DS3234S_YEAR_DEFAULT;

	DS3234S_CS_PORT |= _BV(DS3234S_CS);
}