/*
 * DS3234S.h
 *
 * Created: 2019/04/25 8:51:13
 *  Author: mNEFO
 */ 

#ifndef DS3234S_H_
#define DS3234S_H_

#define DS3234S_CS_PORT PORTB
#define DS3234S_CS PB0

void init_SPI(void);
void init_DS3234S(void);
void DS3234S_read_all(void);
void DS3234S_read_time(void);
void DS3234S_read_date(void);
void DS3234S_write_bcd(uint8_t addr, uint8_t data);
void DS3234S_write_int(uint8_t addr, uint8_t data);
void DS3234S_write_time_bcd(uint8_t sec, uint8_t min, uint8_t hour);
void DS3234S_write_date_bcd(uint8_t day, uint8_t date, uint8_t month, uint8_t year);
void DS3234S_write_time_int(uint8_t sec, uint8_t min, uint8_t hour);
void DS3234S_write_date_int(uint8_t day, uint8_t date, uint8_t month, uint8_t year);
void reset_DS3234S(void);

// dummy data for read data
#define SPI_DUMMY 0x00

// read & write
#define DS3234S_READ  0x00
#define DS3234S_WRITE 0x80

// DS3234S read addresses
// add 0x80 for write address
// in DS3234S_write, add 0x80 automatically
#define DS3234S_SEC_ADDR            0x00
#define DS3234S_MIN_ADDR            0x01
#define DS3234S_HOUR_ADDR           0x02
#define DS3234S_DAY_ADDR            0x03
#define DS3234S_DATE_ADDR           0x04
#define DS3234S_MONTH_ADDR          0x05
#define DS3234S_YEAR_ADDR           0x06
#define DS3234S_ALARM1_SEC_ADDR     0x07
#define DS3234S_ALARM1_MIN_ADDR     0x08
#define DS3234S_ALARM1_HOUR_ADDR    0x09
#define DS3234S_ALARM1_DATE_ADDR    0x0A
#define DS3234S_ALARM2_MIN_ADDR     0x0B
#define DS3234S_ALARM2_HOUR_ADDR    0x0C
#define DS3234S_ALARM2_DATE_ADDR    0x0D
#define DS3234S_CONTROL_ADDR        0x0E
#define DS3234S_CONTROL_STATUS_ADDR 0x0F
#define DS3234S_AGING_OFFSET_ADDR   0x10
#define DS3234S_TEMP_MSB_ADDR       0x11 // raed only
#define DS3234S_TEMP_LSB_ADDR       0x12 // read only
#define DS3234S_DIS_TEMP_CONV_ADDR  0x13
#define DS3234S_SRAM_ADDR           0x18
#define DS3234S_SRAM_DATA_ADDR      0x19

// default set is 2000/01/01 (Sat)
#define DS3234S_SEC_DEFAULT         0b00000000 // '00'
#define DS3234S_MIN_DEFAULT         0b00000000 // '00'
#define DS3234S_HOUR_DEFAULT        0b00000000 // '00'
#define DS3234S_DAY_DEFAULT         0b00000111 // '07'
#define DS3234S_DATE_DEFAULT        0b00000001 // '01'
#define DS3234S_MONTH_DEFAULT       0b00000001 // '01'
#define DS3234S_YEAR_DEFAULT        0b00000000 // '00'
#define DS3234S_ALARM1_SEC_DEFAULT  0b00000000
#define DS3234S_ALARM1_MIN_DEFAULT  0b00000000
#define DS3234S_ALARM1_HOUR_DEFAULT 0b00000000
#define DS3234S_ALARM1_DATE_DEFAULT 0b00000000
#define DS3234S_ALARM2_MIN_DEFAULT  0b00000000
#define DS3234S_ALARM2_HOUR_DEFAULT 0b00000000
#define DS3234S_ALARM2_DATE_DEFAULT 0b00000000

#define DS3234S_CONTROL_EOSC  0b10000000
#define DS3234S_CONTROL_BBSQW 0b01000000
#define DS3234S_CONTROL_CONV  0b00100000
#define DS3234S_CONTROL_RS2   0b00010000
#define DS3234S_CONTROL_RS1   0b00001000
#define DS3234S_CONTROL_INTCN 0b00000100
#define DS3234S_CONTROL_A2IE  0b00000010
#define DS3234S_CONTROL_A1IE  0b00000001

#define DS3234S_STATUS_OSF    0b10000000
#define DS3234S_STATUS_BB32   0b01000000
#define DS3234S_STATUS_CRATE1 0b00100000
#define DS3234S_STATUS_CRATE2 0b00010000
#define DS3234S_STATUS_EN32   0b00001000
#define DS3234S_STATUS_BSY    0b00000100
#define DS3234S_STATUS_A2F    0b00000010
#define DS3234S_STATUS_A1F    0b00000001

#define DS3234S_CENTURY_FLAG 0b10000000
#define DS3234S_AMPM_FLAG    0b01000000
#define DS3234S_PM_FLAG      0b00100000

#define TEMP_UPDATE 5

#endif /* DS3234S_H_ */