/*
 * ZM1500_clock.c
 *
 * Created: 2019/04/24 19:21:58
 * Author : mNEFO
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "ad.h"
#include "common.h"
#include "config.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "pin_interrupt.h"
#include "port.h"
#include "timer.h"
#include "uart.h"
#include "xitoa.h"

#ifndef F_CPU
#define F_CPU 16000000UL
#endif

// digit position 0->1->2->3->4->5->6->7->8->9->10->11
volatile uint8_t global_digit = 0;
// duty count
volatile uint8_t global_duty = 0;
// config eeprom map
volatile CONFIG_DATA global_config_eep_map __attribute__((section(".eeprom")));
// config SRAM data
volatile CONFIG_DATA global_config_data;
// mode: NORMAL or SETTING
// startup must be NORMAL, NORMAL setting is saved in eeprom global_config_data.mode
volatile uint8_t global_mode = MODE_NORMAL;
// setting mode
volatile uint8_t global_setting_mode = MODE_SETTING_DUTY;
// config mode
volatile uint8_t global_config_mode = CONFIG_NONE;
// digit duty
volatile uint8_t global_digit_duty[DIGIT_NUM] = {0};
// digit number
volatile uint8_t global_digit_num[DIGIT_NUM] = {0};
// digit PWM
volatile uint8_t global_digit_UP_dot_pwm[DIGIT_NUM] = {0};
volatile uint8_t global_digit_DP_dot_pwm[DIGIT_NUM] = {0};
volatile uint8_t global_digit_UP_dot_pwm_S = 0;
volatile uint8_t global_digit_DP_dot_pwm_S = 0;
volatile uint8_t global_pwm_direction = UP;
// segment font data, see "common.h"
volatile uint8_t global_font_data[12] =
{
	0b01110111, // zero
	0b00010010, // one
	0b01101011, // two
	0b01011011, // three
	0b00011110, // four
	0b01011101, // five
	0b01111101, // six
	0b00010111, // seven
	0b01111111, // eight
	0b01011111, // nine
	0b00000000, // suppress
	0b00001000  // hyphen
};
// DS3234S raw data
volatile uint8_t global_DS3234S_hour = 0, global_DS3234S_min = 0, global_DS3234S_sec = 0;
volatile uint8_t global_DS3234S_year = 0, global_DS3234S_month = 0, global_DS3234S_date = 0, global_DS3234S_day = 0;
// Light sensor value
volatile uint16_t global_light_sensor = 0;
// last update count
volatile uint32_t global_gps_count = GPS_COUNT;
// record GPS time
volatile GPSTIME global_gpstime = {{0}};

// Watch Dog
uint8_t mcusr_mirror __attribute__ ((section (".noinit")));
void reset_mcusr(void) __attribute__((naked)) __attribute__((section(".init3")));
void reset_mcusr(void)
{
	mcusr_mirror = MCUCSR;
	MCUCSR = 0;
	wdt_disable();
}


int main(void)
{
	cli();
	
	// Initialize port
	init_port();
	// Read eeprom setting
	read_eeprom_setting();
	// Initialize A/D converter
	init_adc();
	// Initialize SPI & DS3234S
	init_SPI();
	init_DS3234S();
	// Initialize timer
	init_timer();
	// Initialize pin interrupt
	init_pin_interrupt();
	// Set xprintf output
	xfunc_out = uart1_put;
	// Initialize uart
	init_uart();
	// Set duty
	init_duty();
	// Start HVPS
	on_hvps();

	sei();
	
	// Start GPS
	if(global_config_data.use_gps == ENABLE)
	{
		start_gps();
	}

	// WDT start
	wdt_enable(WDTO_2S);

	// all reset
	if(bit_is_clear(ENC_PORT, ENC_SW_PIN))
	{
		reset_all_setting();
	}

    while(1);
}

