/*
 * mode.c
 *
 * Created: 2019/05/01 13:27:53
 *  Author: mNEFO
 */ 

#include <avr/io.h>

#include "common.h"
#include "config.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "extern.h"
#include "mode.h"
#include "uart.h"

/*
* Control program data and nixie digits.
* Program data is changed by rotary encoder events.
* Such as rotation, push and long push.
* Nixie digits are changed by timer3 event.
*/

// memory date config digit position
volatile uint8_t date_position = POSITION_CENTURY;

// long push event of date mode
// write DS3234S and eeprom
void longpush_config_date(void)
{
	uint8_t min, hour, day, date, month, year, century;
	
	if(date_position == POSITION_MIN)
	{
		// calculate day 1->Sun, 7->Sat
		century = global_digit_num[0] * 10 + global_digit_num[1];
		year = global_digit_num[2] * 10 + global_digit_num[3];
		month = global_digit_num[4] * 10 + global_digit_num[5];
		date = global_digit_num[6] * 10 + global_digit_num[7];
		day = get_zeller_day(century, year, month, date);
		
		// for DS3234S
		min = ((global_digit_num[10] << 4) | global_digit_num[11]);
		hour = ((global_digit_num[8] << 4) | global_digit_num[9]);
		date = ((global_digit_num[6] << 4) | global_digit_num[7]);
		month = ((global_digit_num[4] << 4) | global_digit_num[5]);
		year = ((global_digit_num[2] << 4) | global_digit_num[3]);
		
		// write eeprom & DS3234S
		eep_update_century(century);
		DS3234S_write_date_bcd(day, date, month, year);
		DS3234S_write_time_bcd(0, min, hour);
		
		global_config_mode = CONFIG_NONE;
		date_position = POSITION_CENTURY;
	}
}

// long push event
void update_mode_enc_longpush(void)
{
	switch(global_mode)
	{
		case MODE_NORMAL:
		switch(global_config_data.mode)
		{
			case MODE_NORMAL_YMDT:
			if(global_config_mode == CONFIG_NONE)
			{
				global_config_mode = CONFIG_ENABLE;
			}
			else
			{
				longpush_config_date();
			}
			break;
			
			case MODE_NORMAL_MDTS:
			if(global_config_mode == CONFIG_NONE)
			{
				global_mode = MODE_SETTING;
			}
			else
			{
				// do nothing
			}
			break;
			
			default:
			break;
		}
		break;
		
		case MODE_SETTING:
		switch(global_setting_mode)
		{
			case MODE_SETTING_DUTY:
			if(global_config_mode == CONFIG_NONE)
			{
				global_mode = MODE_NORMAL;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_LIGHT:
			if(global_config_mode == CONFIG_NONE)
			{
				global_mode = MODE_NORMAL;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_LIGHT_TH:
			if(global_config_mode == CONFIG_NONE)
			{
				global_mode = MODE_NORMAL;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_LIGHT_PW:
			if(global_config_mode == CONFIG_NONE)
			{
				global_mode = MODE_NORMAL;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_GPS:
			if(global_config_mode == CONFIG_NONE)
			{
				global_mode = MODE_NORMAL;
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_LOG:
			if(global_config_mode == CONFIG_NONE)
			{
				global_mode = MODE_NORMAL;
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_TZ:
			if(global_config_mode == CONFIG_NONE)
			{
				global_mode = MODE_NORMAL;
			}
			else
			{
				// do nothing
			}
			break;
			
			default:
			break;
		}
		break;
		
		default:
		break;
	}
}

// encoder switch event of date in config
// set digit position for digit duty
void switch_move_date(void)
{
	switch(date_position)
	{
		case POSITION_CENTURY:
		date_position = POSITION_YEAR;
		break;

		case POSITION_YEAR:
		date_position = POSITION_MONTH;
		break;

		case POSITION_MONTH:
		date_position = POSITION_DATE;
		break;
		
		case POSITION_DATE:
		date_position = POSITION_HOUR;
		break;
		
		case POSITION_HOUR:
		date_position = POSITION_MIN;
		break;
		
		case POSITION_MIN:
		date_position = POSITION_CENTURY;
		global_config_mode = CONFIG_NONE;
		normal_duty();
		break;
		
		default:
		date_position = POSITION_CENTURY;
		global_config_mode = CONFIG_NONE;
		normal_duty();
		break;
	}
}

// encoder switch event of date and time mode
// change dot pwm mode
void switch_change_dot_pwm(void)
{
	if(global_config_data.dot_mode == DOT_OFF)
	{
		global_config_data.dot_mode = DOT_PWM;
	}
	else
	{
		global_config_data.dot_mode++;
	}
}

// encoder switch event
void update_mode_enc_switch(void)
{
	switch(global_mode)
	{
		case MODE_NORMAL:
		switch(global_config_data.mode)
		{
			case MODE_NORMAL_YMDT:
			if(global_config_mode == CONFIG_NONE)
			{
				switch_change_dot_pwm();
			}
			else
			{
				switch_move_date();
			}
			break;
			
			case MODE_NORMAL_MDTS:
			if(global_config_mode == CONFIG_NONE)
			{
				switch_change_dot_pwm();
			}
			else
			{
				// do nothing
			}
			break;
			
			default:
			break;
		}
		break;
		
		case MODE_SETTING:
		switch(global_setting_mode)
		{
			case MODE_SETTING_DUTY:
			if(global_config_mode == CONFIG_NONE)
			{
				global_config_mode = CONFIG_ENABLE;
			}
			else
			{
				eep_update_duty(global_config_data.duty);
				normal_duty();
				global_config_mode = CONFIG_NONE;
			}
			break;
			
			case MODE_SETTING_LIGHT:
			if(global_config_mode == CONFIG_NONE)
			{
				if(global_config_data.use_light_sensor == ENABLE)
				{
					global_config_data.use_light_sensor = DISABLE;
				}
				else
				{
					global_config_data.use_light_sensor = ENABLE;
				}
				eep_update_use_light_sensor(global_config_data.use_light_sensor);
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_LIGHT_TH:
			if(global_config_mode == CONFIG_NONE)
			{
				global_config_mode = CONFIG_ENABLE;
			}
			else
			{
				eep_update_light_sensor_th(global_config_data.light_sensor_th);
				normal_duty();
				global_config_mode = CONFIG_NONE;
			}
			break;
			
			case MODE_SETTING_LIGHT_PW:
			if(global_config_mode == CONFIG_NONE)
			{
				if(global_config_data.light_sensor_hvps == ENABLE)
				{
					global_config_data.light_sensor_hvps = DISABLE;
				}
				else
				{
					global_config_data.light_sensor_hvps = ENABLE;
				}
				eep_update_light_sensor_hvps(global_config_data.light_sensor_hvps);
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_GPS:
			if(global_config_mode == CONFIG_NONE)
			{
				if(global_config_data.use_gps == ENABLE)
				{
					global_config_data.use_gps = DISABLE;
					stop_gps();
				}
				else
				{
					global_config_data.use_gps = ENABLE;
					start_gps();
				}
				eep_update_use_gps(global_config_data.use_gps);
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_LOG:
			if(global_config_mode == CONFIG_NONE)
			{
				if(global_config_data.gps_log == ENABLE)
				{
					global_config_data.gps_log = DISABLE;
				}
				else
				{
					global_config_data.gps_log = ENABLE;
				}
				eep_update_gps_log(global_config_data.gps_log);
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_TZ:
			if(global_config_mode == CONFIG_NONE)
			{
				global_config_mode = CONFIG_ENABLE;
			}
			else
			{
				eep_update_gps_tz(global_config_data.gps_tz);
				normal_duty();
				global_config_mode = CONFIG_NONE;
			}
			break;
			
			default:
			break;
		}
		break;
		
		default:
		break;
	}
}

// encoder decrement event of date in config
// change digita data
void decrement_YMDT_config(void)
{
	uint8_t temp;
	uint8_t last_date = 31;
	
	switch(date_position)
	{
		case POSITION_CENTURY:
		temp = global_digit_num[0] * 10 + global_digit_num[1];
		if(temp == 0)
		{
			global_digit_num[0] = 9;
			global_digit_num[1] = 9;
		}
		else
		{
			temp--;
			global_digit_num[0] = temp / 10;
			global_digit_num[1] = temp - global_digit_num[0] * 10;
		}
		break;

		case POSITION_YEAR:
		temp = global_digit_num[2] * 10 + global_digit_num[3];
		if(temp == 0)
		{
			global_digit_num[2] = 9;
			global_digit_num[3] = 9;
		}
		else
		{
			temp--;
			global_digit_num[2] = temp / 10;
			global_digit_num[3] = temp - global_digit_num[2] * 10;
		}
		break;

		case POSITION_MONTH:
		temp = global_digit_num[4] * 10 + global_digit_num[5];
		if(temp == 1)
		{
			global_digit_num[4] = 1;
			global_digit_num[5] = 2;
		}
		else
		{
			temp--;
			global_digit_num[4] = temp / 10;
			global_digit_num[5] = temp - global_digit_num[4] * 10;
		}
		break;
		
		case POSITION_DATE:
		temp = global_digit_num[6] * 10 + global_digit_num[7];
		if(temp == 1)
		{
			last_date = get_monthend(global_digit_num[0] * 10 + global_digit_num[1], global_digit_num[2] * 10 + global_digit_num[3], global_digit_num[4] * 10 + global_digit_num[5]);
			if(last_date == 31)
			{
				global_digit_num[6] = 3;
				global_digit_num[7] = 1;				
			}
			else if(last_date == 30)
			{
				global_digit_num[6] = 3;
				global_digit_num[7] = 0;
			}
			else if(last_date == 28)
			{
				global_digit_num[6] = 2;
				global_digit_num[7] = 8;
			}
			else if(last_date == 29)
			{
				global_digit_num[6] = 2;
				global_digit_num[7] = 9;
			}
			else
			{
				global_digit_num[6] = 3;
				global_digit_num[7] = 1;
			}
		}
		else
		{
			temp--;
			global_digit_num[6] = temp / 10;
			global_digit_num[7] = temp - global_digit_num[6] * 10;
		}
		break;
		
		case POSITION_HOUR:
		temp = global_digit_num[8] * 10 + global_digit_num[9];
		if(temp == 0)
		{
			global_digit_num[8] = 2;
			global_digit_num[9] = 3;
		}
		else
		{
			temp--;
			global_digit_num[8] = temp / 10;
			global_digit_num[9] = temp - global_digit_num[8] * 10;
		}
		break;
		
		case POSITION_MIN:
		temp = global_digit_num[10] * 10 + global_digit_num[11];
		if(temp == 0)
		{
			global_digit_num[10] = 5;
			global_digit_num[11] = 9;
		}
		else
		{
			temp--;
			global_digit_num[10] = temp / 10;
			global_digit_num[11] = temp - global_digit_num[10] * 10;
		}
		break;
		
		default:
		date_position = POSITION_CENTURY;
		break;
	}
}

// encoder decrement event
void update_mode_enc_decrement(void)
{
	switch(global_mode)
	{
		case MODE_NORMAL:
		switch(global_config_data.mode)
		{
			case MODE_NORMAL_YMDT:
			if(global_config_mode == CONFIG_NONE)
			{
				global_config_data.mode = MODE_NORMAL_MDTS;
				eep_update_mode(global_config_data.mode);
			}
			else
			{
				decrement_YMDT_config();
			}
			break;
			
			case MODE_NORMAL_MDTS:
			if(global_config_mode == CONFIG_NONE)
			{
				global_config_data.mode = MODE_NORMAL_YMDT;
				eep_update_mode(global_config_data.mode);				
			}
			else
			{
				// do nothing
			}
			break;
			
			default:
			break;
		}
		break;
		
		case MODE_SETTING:
		switch(global_setting_mode)
		{
			case MODE_SETTING_DUTY:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_GPS_TZ;
			}
			else
			{
				if(global_config_data.duty == DUTY_MIN)
				{
					global_config_data.duty = DUTY_MAX;
				}
				else
				{
					global_config_data.duty--;
				}
			}
			break;
			
			case MODE_SETTING_LIGHT:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_DUTY;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_LIGHT_TH:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_LIGHT;
			}
			else
			{
				if(global_config_data.light_sensor_th == LIGHT_SENSOR_TH_MIN)
				{
					global_config_data.light_sensor_th = LIGHT_SENSOR_TH_MAX;
				}
				else
				{
					global_config_data.light_sensor_th--;
				}
			}
			break;
			
			case MODE_SETTING_LIGHT_PW:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_LIGHT_TH;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_GPS:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_LIGHT_PW;
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_LOG:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_GPS;
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_TZ:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_GPS_LOG;
			}
			else
			{
				if(global_config_data.gps_tz == GPS_TIMEZONE_MIN)
				{
					global_config_data.gps_tz = GPS_TIMEZONE_MAX;
				}
				else
				{
					global_config_data.gps_tz--;
				}
			}
			break;
			
			default:
			global_setting_mode = MODE_SETTING_DUTY;
			break;
		}
		break;
		
		default:
		break;
	}
}


// encoder increment event of date in config
// change digita data
void increment_YMDT_config()
{
	uint8_t temp;
	uint8_t last_date = 31;
	
	switch(date_position)
	{
		case POSITION_CENTURY:
		temp = global_digit_num[0] * 10 + global_digit_num[1];
		if(temp == 99)
		{
			global_digit_num[0] = 0;
			global_digit_num[1] = 0;
		}
		else
		{
			temp++;
			global_digit_num[0] = temp / 10;
			global_digit_num[1] = temp - global_digit_num[0] * 10;
		}
		break;

		case POSITION_YEAR:
		temp = global_digit_num[2] * 10 + global_digit_num[3];
		if(temp == 99)
		{
			global_digit_num[2] = 0;
			global_digit_num[3] = 0;
		}
		else
		{
			temp++;
			global_digit_num[2] = temp / 10;
			global_digit_num[3] = temp - global_digit_num[2] * 10;
		}
		break;

		case POSITION_MONTH:
		temp = global_digit_num[4] * 10 + global_digit_num[5];
		if(temp == 12)
		{
			global_digit_num[4] = 0;
			global_digit_num[5] = 1;
		}
		else
		{
			temp++;
			global_digit_num[4] = temp / 10;
			global_digit_num[5] = temp - global_digit_num[4] * 10;
		}
		break;
		
		case POSITION_DATE:
		temp = global_digit_num[6] * 10 + global_digit_num[7];
		last_date = get_monthend(global_digit_num[0] * 10 + global_digit_num[1], global_digit_num[2] * 10 + global_digit_num[3], global_digit_num[4] * 10 + global_digit_num[5]);
		if(temp == last_date)
		{
			global_digit_num[6] = 0;
			global_digit_num[7] = 1;
		}
		else
		{
			temp++;
			global_digit_num[6] = temp / 10;
			global_digit_num[7] = temp - global_digit_num[6] * 10;
		}
		break;
		
		case POSITION_HOUR:
		temp = global_digit_num[8] * 10 + global_digit_num[9];
		if(temp == 23)
		{
			global_digit_num[8] = 0;
			global_digit_num[9] = 0;
		}
		else
		{
			temp++;
			global_digit_num[8] = temp / 10;
			global_digit_num[9] = temp - global_digit_num[8] * 10;
		}
		break;
		
		case POSITION_MIN:
		temp = global_digit_num[10] * 10 + global_digit_num[11];
		if(temp == 59)
		{
			global_digit_num[10] = 0;
			global_digit_num[11] = 0;
		}
		else
		{
			temp++;
			global_digit_num[10] = temp / 10;
			global_digit_num[11] = temp - global_digit_num[10] * 10;
		}
		break;
		
		default:
		date_position = POSITION_CENTURY;
		break;
	}	
}


// encoder increment event
void update_mode_enc_increment(void)
{
	switch(global_mode)
	{
		case MODE_NORMAL:
		switch(global_config_data.mode)
		{
			case MODE_NORMAL_YMDT:
			if(global_config_mode == CONFIG_NONE)
			{
				global_config_data.mode = MODE_NORMAL_MDTS;
				eep_update_mode(global_config_data.mode);
			}
			else
			{
				increment_YMDT_config();
			}
			break;
			
			case MODE_NORMAL_MDTS:
			if(global_config_mode == CONFIG_NONE)
			{
				global_config_data.mode = MODE_NORMAL_YMDT;
				eep_update_mode(global_config_data.mode);
			}
			else
			{
				// do nothing
			}
			break;
			
			default:
			break;
		}
		break;
		
		case MODE_SETTING:
		switch(global_setting_mode)
		{
			case MODE_SETTING_DUTY:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_LIGHT;
			}
			else
			{
				if(global_config_data.duty == DUTY_MAX)
				{
					global_config_data.duty = DUTY_MIN;
				}
				else
				{
					global_config_data.duty++;
				}
			}
			break;
			
			case MODE_SETTING_LIGHT:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_LIGHT_TH;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_LIGHT_TH:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_LIGHT_PW;
			}
			else
			{
				if(global_config_data.light_sensor_th == LIGHT_SENSOR_TH_MAX)
				{
					global_config_data.light_sensor_th = LIGHT_SENSOR_TH_MIN;
				}
				else
				{
					global_config_data.light_sensor_th++;
				}
			}
			break;
			
			case MODE_SETTING_LIGHT_PW:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_GPS;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_GPS:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_GPS_LOG;
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_LOG:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_GPS_TZ;
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_TZ:
			if(global_config_mode == CONFIG_NONE)
			{
				global_setting_mode = MODE_SETTING_DUTY;
			}
			else
			{
				if(global_config_data.gps_tz == GPS_TIMEZONE_MAX)
				{
					global_config_data.gps_tz = GPS_TIMEZONE_MIN;
				}
				else
				{
					global_config_data.gps_tz++;
				}
			}
			break;
			
			default:
			global_setting_mode = MODE_SETTING_DUTY;
			break;
		}
		break;
		
		default:
		break;
	}
}

void no_dot(void)
{
	uint8_t i = 0;
	for(i=0; i<DIGIT_NUM; i++)
	{
		global_digit_UP_dot_pwm[i] = DOT_PWM_MIN;
		global_digit_DP_dot_pwm[i] = DOT_PWM_MIN;
	}
}

void show_setting_mode(void)
{
	no_dot();
	global_digit_num[0] = global_setting_mode + 1;
	global_digit_num[1] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[2] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[3] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[4] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[5] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[6] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[7] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[8] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[9] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[10] = NIXIE_CATHODE_SUPPRESS;
	global_digit_num[11] = NIXIE_CATHODE_SUPPRESS;
}

// nixie digit update event of duty in config
void show_setting_duty(void)
{
	global_digit_num[10] = global_config_data.duty / 10;
	global_digit_num[11] = global_config_data.duty - global_digit_num[10] * 10;
}

// nixie digit update event of light sensor threshold in config
void show_setting_light_sentor_th(void)
{
	global_digit_num[9]  = global_config_data.light_sensor_th / 100;
	global_digit_num[10] = (global_config_data.light_sensor_th - global_digit_num[9] * 100) / 10;
	global_digit_num[11] = global_config_data.light_sensor_th - global_digit_num[9] * 100 - global_digit_num[10] * 10;
}

// nixie digit update event of gps timezone in config
void show_setting_gps_tz(void)
{
	// gps_tz is int8 (from -12 to +14)
	if(global_config_data.gps_tz >= 0)
	{
		global_digit_num[9] = NIXIE_CATHODE_SUPPRESS;
		global_digit_num[10] = global_config_data.gps_tz / 10;
		global_digit_num[11] = global_config_data.gps_tz - global_digit_num[10] * 10;
	}
	else
	{
		global_digit_num[9] = NIXIE_CATHODE_HYPHEN;
		global_digit_num[10] = (-global_config_data.gps_tz) / 10;
		global_digit_num[11] = (-global_config_data.gps_tz) - global_digit_num[10] * 10;
	}
}

// nixie digit update event
void update_digit(void)
{
	switch(global_mode)
	{
		case MODE_NORMAL:
		switch(global_config_data.mode)
		{
			case MODE_NORMAL_YMDT:
			if(global_config_mode == CONFIG_NONE)
			{
				global_digit_num[0] = global_config_data.century >> 4;
				global_digit_num[1] = global_config_data.century & 0b00001111;
				global_digit_num[2] = global_DS3234S_year >> 4;
				global_digit_num[3] = global_DS3234S_year & 0b00001111;
				global_digit_num[4] = global_DS3234S_month >> 4;
				global_digit_num[5] = global_DS3234S_month & 0b00001111;
				global_digit_num[6] = global_DS3234S_date >> 4;
				global_digit_num[7] = global_DS3234S_date & 0b00001111;
				global_digit_num[8] = (global_DS3234S_hour & 0b00110000) >> 4;
				global_digit_num[9] = global_DS3234S_hour & 0b00001111;
				global_digit_num[10] = global_DS3234S_min >> 4;
				global_digit_num[11] = global_DS3234S_min & 0b00001111;
				global_digit_UP_dot_pwm[0] = 0;
				global_digit_UP_dot_pwm[1] = 0;
				global_digit_UP_dot_pwm[2] = 0;
				global_digit_UP_dot_pwm[3] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[4] = 0;
				global_digit_UP_dot_pwm[5] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[6] = 0;
				global_digit_UP_dot_pwm[7] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[8] = 0;
				global_digit_UP_dot_pwm[9] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[10] = 0;
				global_digit_UP_dot_pwm[11] = 0;
				global_digit_DP_dot_pwm[0] = 0;
				global_digit_DP_dot_pwm[1] = 0;
				global_digit_DP_dot_pwm[2] = 0;
				global_digit_DP_dot_pwm[3] = 0;
				global_digit_DP_dot_pwm[4] = 0;
				global_digit_DP_dot_pwm[5] = 0;
				global_digit_DP_dot_pwm[6] = 0;
				global_digit_DP_dot_pwm[7] = 0;
				global_digit_DP_dot_pwm[8] = 0;
				global_digit_DP_dot_pwm[9] = global_digit_DP_dot_pwm_S;
				global_digit_DP_dot_pwm[10] = 0;
				global_digit_DP_dot_pwm[11] = 0;
			}
			else
			{
				minimum_duty();
				global_digit_duty[date_position] = DUTY_MAX;
				global_digit_duty[date_position + 1] = DUTY_MAX;
			}
			break;
			
			case MODE_NORMAL_MDTS:
			if(global_config_mode == CONFIG_NONE)
			{
				/*
				global_digit_num[0] = global_DS3234S_month >> 4;
				global_digit_num[1] = global_DS3234S_month & 0b00001111;
				global_digit_num[2] = global_DS3234S_date >> 4;
				global_digit_num[3] = global_DS3234S_date & 0b00001111;
				global_digit_num[4] = (global_DS3234S_hour & 0b00110000) >> 4;
				global_digit_num[5] = global_DS3234S_hour & 0b00001111;
				global_digit_num[6] = NIXIE_CATHODE_SUPPRESS;
				global_digit_num[7] = global_DS3234S_min >> 4;
				global_digit_num[8] = global_DS3234S_min & 0b00001111;
				global_digit_num[9] = NIXIE_CATHODE_SUPPRESS;
				global_digit_num[10] = global_DS3234S_sec >> 4;
				global_digit_num[11] = global_DS3234S_sec & 0b00001111;
				global_digit_UP_dot_pwm[0] = 0;
				global_digit_UP_dot_pwm[1] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[2] = 0;
				global_digit_UP_dot_pwm[3] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[4] = 0;
				global_digit_UP_dot_pwm[5] = 0;
				global_digit_UP_dot_pwm[6] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[7] = 0;
				global_digit_UP_dot_pwm[8] = 0;
				global_digit_UP_dot_pwm[9] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[10] = 0;
				global_digit_UP_dot_pwm[11] = 0;
				global_digit_DP_dot_pwm[0] = 0;
				global_digit_DP_dot_pwm[1] = 0;
				global_digit_DP_dot_pwm[2] = 0;
				global_digit_DP_dot_pwm[3] = 0;
				global_digit_DP_dot_pwm[4] = 0;
				global_digit_DP_dot_pwm[5] = 0;
				global_digit_DP_dot_pwm[6] = global_digit_DP_dot_pwm_S;
				global_digit_DP_dot_pwm[7] = 0;
				global_digit_DP_dot_pwm[8] = 0;
				global_digit_DP_dot_pwm[9] = global_digit_DP_dot_pwm_S;
				global_digit_DP_dot_pwm[10] = 0;
				global_digit_DP_dot_pwm[11] = 0;
				*/
				global_digit_num[0] = NIXIE_CATHODE_SUPPRESS;
				global_digit_num[1] = NIXIE_CATHODE_SUPPRESS;
				global_digit_num[2] = (global_DS3234S_hour & 0b00110000) >> 4;
				global_digit_num[3] = global_DS3234S_hour & 0b00001111;
				global_digit_num[4] = NIXIE_CATHODE_SUPPRESS;
				global_digit_num[5] = global_DS3234S_min >> 4;
				global_digit_num[6] = global_DS3234S_min & 0b00001111;
				global_digit_num[7] = NIXIE_CATHODE_SUPPRESS;
				global_digit_num[8] = global_DS3234S_sec >> 4;
				global_digit_num[9] = global_DS3234S_sec & 0b00001111;
				global_digit_num[10] = NIXIE_CATHODE_SUPPRESS;
				global_digit_num[11] = NIXIE_CATHODE_SUPPRESS;
				global_digit_UP_dot_pwm[0] = 0;
				global_digit_UP_dot_pwm[1] = 0;
				global_digit_UP_dot_pwm[2] = 0;
				global_digit_UP_dot_pwm[3] = 0;
				global_digit_UP_dot_pwm[4] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[5] = 0;
				global_digit_UP_dot_pwm[6] = 0;
				global_digit_UP_dot_pwm[7] = global_digit_UP_dot_pwm_S;
				global_digit_UP_dot_pwm[8] = 0;
				global_digit_UP_dot_pwm[9] = 0;
				global_digit_UP_dot_pwm[10] = 0;
				global_digit_UP_dot_pwm[11] = 0;
				global_digit_DP_dot_pwm[0] = 0;
				global_digit_DP_dot_pwm[1] = 0;
				global_digit_DP_dot_pwm[2] = 0;
				global_digit_DP_dot_pwm[3] = 0;
				global_digit_DP_dot_pwm[4] = global_digit_DP_dot_pwm_S;
				global_digit_DP_dot_pwm[5] = 0;
				global_digit_DP_dot_pwm[6] = 0;
				global_digit_DP_dot_pwm[7] = global_digit_DP_dot_pwm_S;
				global_digit_DP_dot_pwm[8] = 0;
				global_digit_DP_dot_pwm[9] = 0;
				global_digit_DP_dot_pwm[10] = 0;
				global_digit_DP_dot_pwm[11] = 0;				
			}
			else
			{
				// do nothing
			}
			break;
			
			default:
			break;
		}
		break;
		
		case MODE_SETTING:
		switch(global_setting_mode)
		{
			case MODE_SETTING_DUTY:
			if(global_config_mode == CONFIG_NONE)
			{
				show_setting_mode();
				show_setting_duty();
			}
			else
			{
				show_setting_mode();
				show_setting_duty();
				minimum_duty();
				global_digit_duty[10] = DUTY_MAX;
				global_digit_duty[11] = DUTY_MAX;
			}
			break;
			
			case MODE_SETTING_LIGHT:
			if(global_config_mode == CONFIG_NONE)
			{
				show_setting_mode();
				global_digit_num[DIGIT_NUM - 1] = global_config_data.use_light_sensor;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_LIGHT_TH:
			if(global_config_mode == CONFIG_NONE)
			{
				show_setting_mode();
				show_setting_light_sentor_th();
			}
			else
			{
				show_setting_mode();
				show_setting_light_sentor_th();
				minimum_duty();
				global_digit_duty[9] = DUTY_MAX;
				global_digit_duty[10] = DUTY_MAX;
				global_digit_duty[11] = DUTY_MAX;
			}
			break;
			
			case MODE_SETTING_LIGHT_PW:
			if(global_config_mode == CONFIG_NONE)
			{
				show_setting_mode();
				global_digit_num[DIGIT_NUM - 1] = global_config_data.light_sensor_hvps;
			}
			else
			{
				// do nothing
			}
			break;
			
			case MODE_SETTING_GPS:
			if(global_config_mode == CONFIG_NONE)
			{
				show_setting_mode();
				global_digit_num[DIGIT_NUM - 1] = global_config_data.use_gps;
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_LOG:
			if(global_config_mode == CONFIG_NONE)
			{
				show_setting_mode();
				global_digit_num[DIGIT_NUM - 1] = global_config_data.gps_log;
			}
			else
			{
				// do nothing
			}
			break;

			case MODE_SETTING_GPS_TZ:
			if(global_config_mode == CONFIG_NONE)
			{
				show_setting_mode();
				show_setting_gps_tz();
			}
			else
			{
				show_setting_mode();
				show_setting_gps_tz();
				minimum_duty();
				global_digit_duty[9] = DUTY_MAX;
				global_digit_duty[10] = DUTY_MAX;
				global_digit_duty[11] = DUTY_MAX;
			}
			break;
			
			default:
			break;
		}
		break;
		
		default:
		break;
	}
}