/*
 * pin_interrupt.c
 *
 * Created: 2019/04/25 23:32:11
 *  Author: mNEFO
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sfr_defs.h>
#include <avr/wdt.h>

#include "ad.h"
#include "common.h"
#include "config.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "extern.h"
#include "pin_interrupt.h"
#include "uart.h"

// Initialize pin interrupt.
void init_pin_interrupt(void)
{
	// INT4 for GPS 1pps & INT7 for DS3234S
	EIMSK |= _BV(INT4) | _BV(INT7);
	// DS3234S is clock up/down edge, GPS is clock up edge
	EICRB = _BV(ISC40) | _BV(ISC41) | _BV(ISC70);
}

// Sync to DS3234S from pin input GPS 1pps.
// When GPS output 1pps GPS is 3D fixed and time is accurate.
ISR(INT4_vect)
{
	uint8_t temp_hour;
	uint8_t temp_gpstime[6] = {0};

	// update by 10sec @ sec == 5
	// global_gpstime.time is updating @ ISR(USART0_RX_vect).
	if(global_gpstime.time[5] == GPS_DS3234S_SYNC_TIME && global_gpstime.valid == TRUE &&  global_config_data.use_gps == TRUE)
	{
		global_gpstime.valid = FALSE;
		
		// copy gpstime
		temp_gpstime[0] = global_gpstime.time[0];
		temp_gpstime[1] = global_gpstime.time[1];
		temp_gpstime[2] = global_gpstime.time[2];
		temp_gpstime[3] = global_gpstime.time[3];
		temp_gpstime[4] = global_gpstime.time[4];
		temp_gpstime[5] = global_gpstime.time[5];

		// character to num
		CHARNUM_TO_NUM(temp_gpstime[0]);
		CHARNUM_TO_NUM(temp_gpstime[1]);
		CHARNUM_TO_NUM(temp_gpstime[2]);
		CHARNUM_TO_NUM(temp_gpstime[3]);
		CHARNUM_TO_NUM(temp_gpstime[4]);
		CHARNUM_TO_NUM(temp_gpstime[5]);
		
		// adjust timezone
		temp_hour = temp_gpstime[0] * 10 + temp_gpstime[1];
		temp_hour += global_config_data.gps_tz;

		// check 24hr overflow by timezone adjustment
		if(temp_hour >= 24)
		{
			temp_hour -= 24;
		}
		
		// return to digit
		temp_gpstime[0] = temp_hour / 10;
		temp_gpstime[1] = temp_hour - temp_gpstime[0] * 10;

		// adjust 1pps timing
		// adjusted second must be 0 - 8, 9 is not good
		temp_gpstime[5]++;

		// sync to DS3234S
		DS3234S_write_bcd(DS3234S_SEC_ADDR,  (temp_gpstime[4] << 4) | temp_gpstime[5]);
		DS3234S_write_bcd(DS3234S_MIN_ADDR,  (temp_gpstime[2] << 4) | temp_gpstime[3]);
		DS3234S_write_bcd(DS3234S_HOUR_ADDR, (temp_gpstime[0] << 4) | temp_gpstime[1]);

		// update flag
		global_gps_count = 0;
		
		// signal LED
		on_gps_led();
	}
}

// DS3234S 1Hz int(clock up/down edge 0.5Hz)
// Monitor: DS3234S data, mode, light sensor, all sec, HVPS.
ISR(INT7_vect)
{
	static uint32_t sec_count_sub = 0;

	// update DS3234S data
	DS3234S_read_all();
	
	// update pwm direction
	if((global_DS3234S_sec & 0b00000001) == 0) // sec is even
	{
		global_pwm_direction = UP;
	}
	else // sec is odd
	{
		global_pwm_direction = DOWN;
	}

	// check light sensor & do dimmer
	check_dimmer();

	// update operation time by 2hour
	sec_count_sub++;
	if(sec_count_sub > SEC_UPDATE_COUNT)
	{
		sec_count_sub = 0;
		eep_update_sec_count(global_config_data.sec_count + SEC_UPDATE_INCREMENT);
	}
	
	// suppress LED
	off_gps_led();

	// reset WDT
	wdt_reset();
}