/*
 * timer.c
 *
 * Created: 2019/04/27 17:12:43
 *  Author: mNEFO
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sfr_defs.h>
#include <avr/wdt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ad.h"
#include "common.h"
#include "DS3234S.h"
#include "eeprom.h"
#include "extern.h"
#include "mode.h"
#include "timer.h"
#include "uart.h"

void init_timer(void)
{
	// timer0 CTC interrupt @ 10 usec
	TIMSK |= _BV(OCIE0);
	TCCR0 = _BV(WGM01) | _BV(CS00); // 1/1
	OCR0 = 160;

	// timer1 pwm
	// enable pwm(16bit timer) for dot
	TCCR1A = _BV(COM1A1) | _BV(COM1B1) | _BV(COM1C1) | _BV(WGM10); // 8 bit fast PWM
	TCCR1B = _BV(WGM12) | _BV(CS10);
	OCR1A = 0;
	OCR1B = 0;
	OCR1C = 0;

	// timer2 control dot
	TIMSK |= _BV(OCIE2);
	TCCR2 = _BV(CS22); // 1/256
	//TCCR2 = _BV(CS20) | _BV(CS22); // 1/1024

	// timer3 for SW interrupts(1ms:1000Hz)
	ETIMSK |= _BV(OCIE3A);
	TCCR3A = 0;
	TCCR3B = _BV(WGM32) | _BV(CS30); // 1/1
	OCR3A = 16000;
}

// timer0 control nixie
ISR(TIMER0_COMP_vect)
{
	// turn on digit until global_duty is less than global_digit_duty[]
	if(global_duty < global_digit_duty[global_digit])
	{
		// ZM1500 has 12 digits by PA0-PA7 and PF0-PF3
		if(global_digit < NIXIE_ANODE_DVIDE) // PA0-PA7
		{
			NIXIE_ANODE_PORT_0 |= _BV(global_digit);
			NIXIE_CATHODE_NUM_PORT = global_font_data[global_digit_num[global_digit]];
			NIXIE_CATHODE_UP_DOT = global_digit_UP_dot_pwm[global_digit];
			NIXIE_CATHODE_DP_DOT = global_digit_DP_dot_pwm[global_digit];
		}
		else // PF0-PF3
		{
			NIXIE_ANODE_PORT_1 |= _BV(global_digit - NIXIE_ANODE_DVIDE);
			NIXIE_CATHODE_NUM_PORT = global_font_data[global_digit_num[global_digit]];
			NIXIE_CATHODE_UP_DOT = global_digit_UP_dot_pwm[global_digit];
			NIXIE_CATHODE_DP_DOT = global_digit_DP_dot_pwm[global_digit];
		}
		global_duty++;
	}
	// turn off digit when global_duty is larger than global_digit_duty[]
	else if(global_duty < DUTY_CYCLE)
	{
		// ZM1500 has 12 digits by PA0-PA7 and PF0-PF3
		if(global_digit < NIXIE_ANODE_DVIDE) // PA0-PA7
		{
			NIXIE_ANODE_PORT_0 &= (uint8_t)~_BV(global_digit);
			NIXIE_CATHODE_NUM_PORT = NIXIE_OUTPUT_N;
			NIXIE_CATHODE_UP_DOT = 0;
			NIXIE_CATHODE_DP_DOT = 0;
		}
		else // PF0-PF3
		{
			NIXIE_ANODE_PORT_1 &= (uint8_t)~_BV(global_digit - NIXIE_ANODE_DVIDE);
			NIXIE_CATHODE_NUM_PORT = NIXIE_OUTPUT_N;
			NIXIE_CATHODE_UP_DOT = 0;
			NIXIE_CATHODE_DP_DOT = 0;
		}
		global_duty++;
	}
	else
	{
		global_duty = 0;
		global_digit++;
		if(global_digit >= DIGIT_NUM){global_digit = 0;}
	}
}

// control nixie digit & dot
ISR(TIMER2_COMP_vect)
{
	static uint8_t temp_pwm = 0;
	
	// dot control
	if(global_config_data.dot_mode == DOT_PWM) // pwm up/down
	{
		// direction is controlled @ DS3234S 1Hz interrupt
		if(global_pwm_direction == UP) // brightness up
		{
			if(temp_pwm < DOT_PWM_MAX)
			{
				temp_pwm++;
				temp_pwm++;
				//temp_pwm++;
				global_digit_UP_dot_pwm_S = temp_pwm;
				global_digit_DP_dot_pwm_S = temp_pwm;
			}
		}
		else // brightness down
		{
			if(temp_pwm > DOT_PWM_MIN)
			{
				temp_pwm--;
				temp_pwm--;
				//temp_pwm--;
				global_digit_UP_dot_pwm_S = temp_pwm;
				global_digit_DP_dot_pwm_S = temp_pwm;
			}			
		}
	}
	else if(global_config_data.dot_mode == DOT_BLINK) // normal on/off
	{
		// direction is controlled @ DS3234S 1Hz interrupt
		if(global_pwm_direction == UP) // on
		{
			global_digit_UP_dot_pwm_S = DOT_PWM_MAX;
			global_digit_DP_dot_pwm_S = DOT_PWM_MAX;
		}
		else // off
		{
			global_digit_UP_dot_pwm_S = DOT_PWM_MIN;
			global_digit_DP_dot_pwm_S = DOT_PWM_MIN;
		}		
	}
	else if(global_config_data.dot_mode == DOT_ON) // always on
	{
		global_digit_UP_dot_pwm_S = DOT_PWM_MAX;
		global_digit_DP_dot_pwm_S = DOT_PWM_MAX;
	}
	else if(global_config_data.dot_mode == DOT_OFF) // always off
	{
		global_digit_UP_dot_pwm_S = DOT_PWM_MIN;
		global_digit_DP_dot_pwm_S = DOT_PWM_MIN;
	}
}

// watch various, encoder switch, encoder ccw cw and digit. 
// SW input configure clock
ISR(TIMER3_COMPA_vect)
{
	// Encoder and Switch
	static uint8_t enc_sw_old = _BV(ENC_SW_PIN); // ENC SW old
	static uint8_t enc_sw;                       // removed chattering ENC SW
	uint8_t enc_sw_new;                          // ENC SW new
	static uint8_t enc;                          // encoder
	static uint16_t long_push = 0;               // long push & skip flag
	
	uint8_t temp;

	// digit update
	update_digit();

	// check ENC SW
	enc_sw_new = ENC_PORT & _BV(ENC_SW_PIN);
	temp = ((uint8_t)~(enc_sw_old ^ enc_sw_new) & (enc_sw ^ enc_sw_new)); // twice same & afterward is different
	enc_sw_old = enc_sw_new;
	enc_sw ^= temp;

	// check ENC
	// http://elm-chan.org/docs/tec/te04.html
	temp = 0;
	if((ENC_PORT & _BV(ENC_A_PIN))){temp = 1;}  // phase A
	if((ENC_PORT & _BV(ENC_B_PIN))){temp += 2;} // phase B
	enc = (enc << 2) + temp;
	enc &= 15;

	// check long push
	if(bit_is_clear(ENC_PORT, ENC_SW_PIN))
	{
		long_push++;
	}
	else
	{
		if(long_push < LONG_PUSH_COUNT)
		{
			long_push = 0;
		}
	}

	// long push operation
	if(long_push > LONG_PUSH_COUNT && bit_is_set(ENC_PORT, ENC_SW_PIN))
	{
		long_push = 0;
		toggle_gps_led();
		update_mode_enc_longpush();
		return;
	}

	// ENC SW operation
	if((enc_sw == 0) && (enc_sw_old == _BV(ENC_SW_PIN)))
	{
		toggle_gps_led();
		update_mode_enc_switch();
		return;
	}

	// ENC rotate operation
	switch(enc)
	{
		case 0x7: // decrement
		toggle_gps_led();
		update_mode_enc_decrement();
		return;
		
		case 0xd: // increment
		toggle_gps_led();
		update_mode_enc_increment();
		return;
			
		default:
		break;
	}

	// reset WDT
	wdt_reset();
}