/*
 * common.c
 *
 * Created: 2019/05/06 16:18:34
 *  Author: mNEFO
 */ 

#include <avr/boot.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "common.h"
#include "extern.h"
#include "uart.h"

// LED
void on_led(void)
{
	sbi(LED_PORT, LED);
}

void off_led(void)
{
	cbi(LED_PORT, LED);
}

void toggle_led(void)
{
	rbi(LED_PORT, LED);
}

BOOL check_xmodem_data(void)
{
	static uint8_t block_num = XMODEM_BLOCK_NUM_START;
	uint8_t check_sum = 0;
	uint8_t i;
	
	// check block number
	if(block_num != XMODEM_BLOCK_DATA_NUM){return FALSE;}
	
	// check complement
	if((XMODEM_BLOCK_DATA_NUM | XMODEM_BLOCK_DATA_NUM_COMP) != 255){return FALSE;}

	// checksum ** this is not CRC
	for(i=XMODEM_DATA_START; i<=XMODEM_DATA_END; i++)
	{
		check_sum += global_Xmodem_buf[i];
	}
	if(check_sum != XMODEM_CHECKSUM){return FALSE;}

	// update block number. overflow 255->0
	if(block_num == XMODEM_BLOCK_NUM_MAX)
	{
		block_num = 0;
	}
	else
	{
		block_num++;
	}
	
	return TRUE;
}

void write_flash(void)
{
	uint8_t sreg;
	uint16_t i;
	uint16_t one_word = 0;
	
	// copy data from Xmodem buffer
	for(i=XMODEM_DATA_START; i<=XMODEM_DATA_END; i++)
	{
		if(global_flash_buf_count == 0)
		{
			global_flash_buf[i-XMODEM_DATA_START] = global_Xmodem_buf[i];

		}
		else if(global_flash_buf_count == 1)
		{
			global_flash_buf[128+i-XMODEM_DATA_START] = global_Xmodem_buf[i];
		}
	}
	global_flash_buf_count++;
	
	// write flash @ 256 byte = SPM_PAGESIZE
	if(global_flash_buf_count == 2)
	{
		toggle_led();
		global_flash_buf_count = 0;
		sreg = SREG;
		cli();
		eeprom_busy_wait();
		boot_page_erase(global_flash_addr);
		boot_spm_busy_wait();
		for(i=0; i<SPM_PAGESIZE; i+=2)
		{
			one_word = (uint16_t)global_flash_buf[i];
			one_word |= (uint16_t)(global_flash_buf[i+1] << 8);
			boot_page_fill(global_flash_addr + i, one_word);
		}
		boot_page_write(global_flash_addr);
		boot_spm_busy_wait();
		boot_rww_enable();
		SREG = sreg;
		global_flash_addr += SPM_PAGESIZE;
	}
}

void write_flash_last(void)
{
	uint8_t sreg;
	uint16_t i;
	uint16_t one_word = 0;
	uint8_t padding_check = TRUE;
	
	// transfer end odd
	if((XMODEM_BLOCK_DATA_NUM & 0b00000001) == 1)
	{
		// check 127->0, 128->255 is dummy
		for(i=127; i>=0; i--)
		{
			if(global_flash_buf[i] == _EOF && padding_check == TRUE)
			{
				global_flash_buf[i] = 0xff;
			}
			else
			{
				global_flash_buf[i+1] = 0x1a; 
				padding_check = FALSE;
			}
		}
		for(i=128; i<256; i++)
		{
			global_flash_buf[i] = 0xff;
		}
	}
	// transfer end even
	else
	{
		global_flash_addr -= SPM_PAGESIZE;
		// check 255->128, 0->127 do nothing
		for(i=255; i>=128; i--)
		{
			if(global_flash_buf[i] == _EOF && padding_check == TRUE)
			{
				global_flash_buf[i] = 0xff;
			}
			else
			{
				global_flash_buf[i+1] = 0x1a;
				padding_check = FALSE;
			}
		}
	}
	
	// write flash @ 256 byte = SPM_PAGESIZE
	sreg = SREG;
	cli();
	eeprom_busy_wait();
	boot_page_erase(global_flash_addr);
	boot_spm_busy_wait();
	for(i=0; i<SPM_PAGESIZE; i+=2)
	{
		one_word = (uint16_t)global_flash_buf[i];
		one_word |= (uint16_t)(global_flash_buf[i+1] << 8);
		boot_page_fill(global_flash_addr + i, one_word);
	}
	boot_page_write(global_flash_addr);
	boot_spm_busy_wait();
	boot_rww_enable();
	SREG = sreg;
}

// GPS control
/*
void start_gps(void)
{
	GPS_PS_PORT |= _BV(GPS_PS);

	xfunc_out = uart0_put; // change xitoa function to uart0_put
	xprintf(PSTR("$PMTK101*32\n")); // hot start
	xfunc_out = uart1_put; // // change xitoa function to uart1_put
}

void stop_gps(void)
{
	//GPS_PS_PORT &= (uint8_t)~_BV(GPS_PS);

	xfunc_out = uart0_put; // change xitoa function to uart0_put
	xprintf(PSTR("$PMTK161,0*28\n")); // standby
	xfunc_out = uart1_put; // // change xitoa function to uart1_put
}

void reset_gps(void)
{
	GPS_PS_PORT |= _BV(GPS_PS);

	xfunc_out = uart0_put; // change xitoa function to uart0_put
	xprintf(PSTR("$PMTK104*37\n")); // full cold start(factory reset)
	xfunc_out = uart1_put; // change xitoa function to uart1_put
}
*/

// runapp
void runapp(void)
{
	uint8_t temp;
	temp = MCUCR;
	MCUCR = (temp | _BV(IVCE));
	MCUCR = (temp & ~_BV(IVSEL) & ~_BV(IVCE));
	MCUCSR = 0;
	asm("jmp 0");
}