/*
 * bootloader.c
 *
 * Created: 2019/05/05 23:45:08
 * Author : mNEFO
 */ 

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <stdlib.h>

#include "uart.h"
#include "xitoa.h"
#include "common.h"
#include "port.h"
#include "timer.h"

#ifndef F_CPU
#define F_CPU 16000000UL
#endif

volatile uint8_t global_Xmodem_buf[XMODEM_BLOCK_SIZE] = {0};
volatile uint8_t global_first_nak_transferred = FALSE;
volatile uint8_t global_Xmodem = FALSE;
volatile uint8_t global_packet_count = 0;
volatile uint8_t global_header_flag = FALSE;
volatile uint32_t global_flash_addr = 0;
volatile uint8_t global_flash_buf[SPM_PAGESIZE] = {0xff};
volatile uint8_t global_flash_buf_count = 0;
volatile uint8_t global_Xmodem_complete = FALSE;

// Watch Dog
uint8_t mcusr_mirror __attribute__ ((section (".noinit")));
void reset_mcusr(void) __attribute__((naked)) __attribute__((section(".init3")));
void reset_mcusr(void)
{
	mcusr_mirror = MCUCSR;
	MCUCSR = 0;
	wdt_disable();
}

int main(void)
{
	uint8_t temp;

	// change vector address
	temp = MCUCR;
	MCUCR = (temp | _BV(IVCE));
	MCUCR = ((temp | _BV(IVSEL)) & ~_BV(IVCE));

	// HW reset and check 0x0000(userapp) flash	
	if (!(mcusr_mirror & _BV(EXTRF)) && pgm_read_byte_near(0x0000) != 0xff)
	{
		runapp();
	}
	
	// Initialize port
	init_port();
	
	// init timer
	init_timer();
	
	// set xprintf output
	xfunc_out = uart1_put;
	
	// init uart
	init_uart();
	
	sei();
	
	// stop gps
	/*
	_delay_ms(500);
	stop_gps();
	_delay_ms(500);
	*/

	xprintf(PSTR("BOOTLOADER MODE\n"));
	xprintf(PSTR("Newline is LF.\n"));
	xprintf(PSTR("Type F to flash.\n"));
	xprintf(PSTR("After flash reconnect power.\n"));

    while(1);
}

