/*
 * port.c
 *
 * Created: 2019/04/24 19:25:49
 *  Author: mNEFO
 */ 

#include <avr/io.h>
#include "port.h"

void init_port(void)
{
/* PORT
** IN :init input    = 0
** OUT:init output   = 1
** PD :init pulldown = 0
** PU :init pullup   = 1 
** NC :Not Connected output & pullup
*/

/* PORT A
** PA0 OUT PD NIXIE1 anode
** PA1 OUT PD NIXIE2 anode
** PA2 OUT PD NIXIE3 anode
** PA3 OUT PD NIXIE4 anode
** PA4 OUT PD NIXIE5 anode
** PA5 OUT PD NIXIE6 anode
** PA6 OUT PD NIXIE7 anode
** PA7 OUT PD NIXIE8 anode
*/
	DDRA  = 0b11111111;
	PORTA = 0b00000000;

/* PORT B
** PB0 OUT PD DS3234S CS  (SPI_SS)
** PB1 OUT PD DS3234S SCK (SPI_SCK) & ISP
** PB2 OUT PD DS3234S DIN (SPI_MOSI)
** PB3 IN  PD DS3234S DOUT(SPI_MISO)
** PB4 NC
** PB5 NC
** PB6 OUT PD Nixie DP dot(PWM OC1B)
** PB7 OUT PD Nixie UP dot(PWM OC1C)
*/
	DDRB  = 0b11110111;
	PORTB = 0b00110000;

/* PORT C
** PC0 OUT PD NIXIE a segment
** PC1 OUT PD NIXIE b segment
** PC2 OUT PD NIXIE f segment
** PC3 OUT PD NIXIE g segment
** PC4 OUT PD NIXIE c segment
** PC5 OUT PD NIXIE e segment
** PC6 OUT PD NIXIE d segment
** PC7 NC
*/
	DDRC  = 0b11111111;
	PORTC = 0b10000000;

/* PORT D
** PD0 NC
** PD1 NC
** PD2 IN PD RXD(FT230XS TXD)
** PD3 IN PD TXD(FT230XS RXD)
** PD4 NC
** PD5 IN PU ENC A
** PD6 IN PU ENC SW
** PD7 IN PU ENC B
*/
	DDRD  = 0b00010011;
	PORTD = 0b11110011;

/* PORT E
** PE0 PDI(ISP) & IN PD RXD (GPS TXD)
** PE1 PDO(ISP) & IN PD TXD (GPS RXD)
** PE2 OUT PD GPS SW
** PE3 OUT PU HVPS SW(pullup is off)
** PE4 IN PU INT4(GPS 1pps)
** PE5 OUT PU INT5(GPS RST)
** PE6 IN PU INT6(GPS 32kHz)
** PE7 IN PU INT7(DS3234S INT)
*/
	DDRE  = 0b00001100;
	PORTE = 0b11111000;

/* PORT F
** PF0 OUT PD NIXIE9  anode
** PF1 OUT PD NIXIE10 anode
** PF2 OUT PD NIXIE11 anode
** PF3 OUT PD NIXIE12 anode
** PF4 NC
** PF5 NC
** PF6 NC
** PF7 IN  PD (ADC7)Light Sensor
*/
	DDRF  = 0b01111111;
	PORTF = 0b01110000;
/* PORT G
** PG0 OUT PD GPS STATUS LED(BLUE)
** PG1 NC
** PG2 NC
** PG3 NC
** PG4 NC
*/
	DDRG  = 0b00011111;
	PORTG = 0b00011110;
}
