/*
 * timer.c
 *
 * Created: 2019/05/15 20:39:16
 *  Author: mNEFO
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/boot.h>

#include "common.h"
#include "extern.h"
#include "uart.h"

#define WAIT_COUNT 3

void init_timer(void)
{
	// timer1 128ms interrupt
	// 16*10^6(CPU clock) / (2 * N(clock select = 64) * (1 + OCR1A) [Hz] = 0.128 [sec] 
	TIMSK |= _BV(OCIE1A);
	TCCR1A = 0;
	TCCR1B = _BV(WGM12) | _BV(CS11) | _BV(CS10); // clock select = 64
	OCR1A = 15999;
}

ISR(TIMER1_COMPA_vect)
{
	static uint8_t wait = 0;

	wait++;
	if(global_Xmodem == TRUE && wait == WAIT_COUNT)
	{
		wait = 0;
		if(global_first_nak_transferred == FALSE)
		{
			global_header_flag = TRUE;
			global_packet_count = 0;
			uart1_put(_NAK);
		}
		if(global_first_nak_transferred == TRUE && global_packet_count == XMODEM_BLOCK_SIZE)
		{
			if(check_xmodem_data() == TRUE)
			{
				write_flash();
				global_header_flag = TRUE;
				global_packet_count = 0;
				uart1_put(_ACK);			
			}
			else
			{
				global_header_flag = TRUE;
				global_packet_count = 0;
				uart1_put(_NAK);
			}
		}
	}
}